const express = require('express')
const app = express()
const jwt = require('jsonwebtoken')
const secret = require('./secret.js')

app.use(express.json())

// 记录refreshToken
let refreshTokens = []

// 允许跨域请求
app.use((req, res, next) => {
    res.set({
        'Access-Control-Allow-Origin': '*',
        'Access-Control-Allow-Headers': 'X-Requested-With,Content-Type',
        'Access-Control-Allow-Methods': 'GET, POST, PUT, DELETE, OPTIONS'
    })
    next()
})

// 登录
app.post('/login', (req, res) => {
    // 取得前端请求中上行的用户名信息
    const username = req.body.username
    // 组装用户信息并生成 accessToken 返回客户端
    // 这里只是举例将用户信息存入 JWT 的 payload, 实际的应用场景中可任意扩展.
    const user = { name: username }
    const accessToken = generateAccessToken({ user })

    // 刷新accessToken使用的refreshToken
    const refreshToken = jwt.sign({ user }, secret.REFRESH_TOKEN_SECRET)
    refreshTokens.push(refreshToken)

    res.json({ accessToken, refreshToken })
})

// 使用 ACCESS_TOKEN_SECRET 对用户信息进行签名, 返回JWT(AccessToken)
// accessToken 10秒后失效, 实际场景根据需要设定. 类似session的超时时间
function generateAccessToken(payload) {
    return jwt.sign(payload, secret.ACCESS_TOKEN_SECRET, { expiresIn: '10s' })
}

// 刷新accessToken
app.post('/token', function (req, res) {
    const refreshToken = req.body.token
    if (!refreshToken) return res.sendStatus(401)
    if (!refreshTokens.includes(refreshToken)) return res.sendStatus(403)
    jwt.verify(refreshToken, secret.REFRESH_TOKEN_SECRET, (err, payload) => {
        if (err) return res.sendStatus(403)
        const accessToken = generateAccessToken({ user: payload.user })
        res.json({ accessToken })
    })
})

// 退出登录
app.delete('/logout', function (req, res) {
    console.log(req.body.token)
    refreshTokens = refreshTokens.filter(token => token !== req.body.token)
    console.log(refreshTokens)
    res.sendStatus(204)
})

app.listen(9000, () => {
    console.log('Auth Server start on port 9000')
})