const express = require('express')
const app = express()

// 引入第3方库(jsonwebtoken)
const jwt = require('jsonwebtoken')

// 引入密钥文件
const secret = require('./secret.js')

// 使用 json 中间件处理前端上行的 json 数据
app.use(express.json())

// 指定 public 文件夹为前端静态资源
app.use(express.static('public'))

// 这里注册了一个用于鉴权的中间件
// 对前端发来的所有请求都会经此中间件处理
app.use(function(req, res, next) {
    // 从请求头(headers)中获取授权信息(格式为 'Bearer token' 的字符串), 参见 
    const authHeader = req.headers['authorization']
    // 取得授权信息中的token部分
    const token = authHeader && authHeader.split(' ')[1]
    
    // 若无法获得token直接返回401
    if (!token) return res.sendStatus(401)

    // 检验token的有效性 
    // 若token无效则返回403, 否则将从token中取得的用户信息放入request以便后续使用
    // 此处使用了secret.js中存储的ACCESS_TOKEN_SECRET
    jwt.verify(token, secret.ACCESS_TOKEN_SECRET, (err, payload) => {
        if (err) return res.sendStatus(403)
        req.user = payload.user
        next()
    })
})

// 监听前端 /getMessage 请求
app.post('/getMessage', (req, res) => {
    // req.user为token中解码得到的用户信息, 参见第32行
    res.end(`Hi, ${ req.user.name }, ${new Date().toTimeString()}`)
})

// 开始监听 3000 端口 
app.listen(3000, () => {
    console.log('Server start on port 3000')
})